$ifndef _HPWM10_INC_
$define _HPWM10_INC_
'
' Note that these macros use FSR0L/H and INDF0 in order to achieve bankless operation
' Version 2 that takes into account that the SFRs for CCP3 and CCP4 are in RAM bank 15
'
' Configure CCP1 as PWM with a resolution of 10-bits (0 - 1023)
'
' Turn on Timer2 with a Prescaler value of 1:1
' Set PWM resolution to 10-bits
' Reset the CCPR1L register
' Turn on PWM Module 1 by setting bits 2 and 3 of CCP1CON
' Enable the PWM1 output pin
$define OpenAnalog1()   '
    T2CON = %00000100   '
    PR2 = 255           '
    CCPR1L = 0          '
    CCP1CON = %00001100 '
    Output PORTC.2
'
' Configure CCP2 as PWM with a resolution of 10-bits (0 - 1023)
' Turn on Timer2 with a Prescaler value of 1:1
' Set PWM resolution to 10-bits
' Reset the CCPR2L register
' Turn on PWM Module 2 by setting bits 2 and 3 of CCP2CON
' Enable the PWM2 output pin
'
$define OpenAnalog2()   '
    T2CON = %00000100   '
    PR2 = 255           '
    CCPR2L = 0          '
    CCP2CON = %00001100 '
    Output PORTC.1

$ifdef _CCP3CON                     ' Does the device contain the CCP3CON register?
'
' Configure CCP3 as PWM with a resolution of 10-bits (0 - 1023)
' Turn on Timer2 with a Prescaler value of 1:1
' Set PWM resolution to 10-bits
' Reset the CCPR3L register
' Turn on PWM Module 3 by setting bits 2 and 3 of CCP3CON
' Enable the PWM3 output pin
'
$define OpenAnalog3()   '
    T2CON = %00000100   '
    PR2 = 255           '
    CCPR3L = 0          '
    CCP3CON = %00001100 '
    Output PORTB.5
$endif  ' _CCP3CON

$ifdef _CCP4CON                     ' Does the device contain the CCP4CON register?
'
' Configure CCP4 as PWM with a resolution of 10-bits (0 - 1023)
' Turn on Timer2 with a Prescaler value of 1:1
' Set PWM resolution to 10-bits
' Reset the CCPR4L register
' Turn on PWM Module 4 by setting bits 2 and 3 of CCP4CON
' Enable the PWM4 output pin
'
$define OpenAnalog4()   '
    T2CON = %00000100   '
    PR2 = 255           '
    CCPR4L = 0          '
    CCP4CON = %00001100 '
    Output PORTB.0
$endif  ' _CCP4CON

'---------------------------------------------------------
$define WriteAnalog1(pDutyCycle) _mWriteAnalog1 pDutyCycle

_mWriteAnalog1 Macro pDutyCycle
    #if(Prm_1 == Byte)
        Lfsr 0,(pDutyCycle)
        Movlw 207           ' CCP1CON = CCP1CON & %11001111
        Andwf CCP1CON,f
        Swapf INDF0,w       ' WREG = pDutyCycle << 4
        Andlw 48            ' WREG = WREG & %00110000
        Iorwf CCP1CON,f     ' CCP1CON = CCP1CON | WREG
        Bcf STATUS,0        ' CCPR1L = pDutyCycle >> 2
        Rrcf INDF0,w
        Bcf STATUS,0
        Rrcf WREG,f
        Movwf CCPR1L
        Exitm
    #endif
    #if(Prm_1 == Word) || (Prm_1 == Dword)
        Lfsr 0,(pDutyCycle)
        Movlw 207           ' CCP1CON = CCP1CON & %11001111
        Andwf CCP1CON,f
        Swapf POSTINC0,w    ' WREG = pDutyCycle << 4
        Andlw 48            ' WREG = WREG & %00110000
        Iorwf CCP1CON,f     ' CCP1CON = CCP1CON | WREG

        Rrcf POSTDEC0,w     ' CCPR1L = pDutyCycle >> 2
        Movwf PRODL,0
        Rrcf INDF0,w
        Movwf CCPR1L
        Rrcf PRODL,f
        Rrcf CCPR1L,f
        Exitm
    #endif
    #if(Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32)
        Movlw 207
        Andwf CCP1CON,f
        Movlw ((pDutyCycle << 4) & 48)
        Iorwf CCP1CON,f
        Movlw (pDutyCycle >> 2)
        Movwf CCPR1L
        Exitm
    #endif
Endm
'---------------------------------------------------------
$define WriteAnalog2(pDutyCycle) _mWriteAnalog2 pDutyCycle

_mWriteAnalog2 Macro pDutyCycle
    #if(Prm_1 == Byte)
        Lfsr 0,(pDutyCycle)
        Movlw 207           ' CCP2CON = CCP2CON & %11001111
        Andwf CCP2CON,f
        Swapf INDF0,w       ' WREG = pDutyCycle << 4
        Andlw 48            ' WREG = WREG & %00110000
        Iorwf CCP2CON,f     ' CCP2CON = CCP2CON | WREG
        Bcf STATUS,0        ' CCPR2L = pDutyCycle >> 2
        Rrcf INDF0,w
        Bcf STATUS,0
        Rrcf WREG,f
        Movwf CCPR2L
        Exitm
    #endif
    #if(Prm_1 == Word) || (Prm_1 == Dword)
        Lfsr 0,(pDutyCycle)
        Movlw 207           ' CCP2CON = CCP2CON & %11001111
        Andwf CCP2CON,f
        Swapf POSTINC0,w    ' WREG = pDutyCycle << 4
        Andlw 48            ' WREG = WREG & %00110000
        Iorwf CCP2CON,f     ' CCP2CON = CCP2CON | WREG

        Rrcf POSTDEC0,w     ' CCPR2L = pDutyCycle >> 2
        Movwf PRODL,0
        Rrcf INDF0,w
        Movwf CCPR2L
        Rrcf PRODL,f
        Rrcf CCPR2L,f
        Exitm
    #endif
    #if(Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32)
        Movlw 207
        Andwf CCP2CON,f
        Movlw ((pDutyCycle << 4) & 48)
        Iorwf CCP2CON,f
        Movlw (pDutyCycle >> 2)
        Movwf CCPR2L
        Exitm
    #endif
Endm

$ifdef _CCP3CON                     ' Does the device contain the CCP3CON register?
'---------------------------------------------------------
$define WriteAnalog3(pDutyCycle) _mWriteAnalog3 pDutyCycle

_mWriteAnalog3 Macro pDutyCycle
    Movlb 15                ' CCP3 SFRs are in RAM bank 15
    #if(Prm_1 == Byte)
        Lfsr 0,(pDutyCycle)
        Movlw 207           ' CCP3CON = CCP3CON & %11001111
        Andwf CCP3CON,f
        Swapf INDF0,w       ' WREG = pDutyCycle << 4
        Andlw 48            ' WREG = WREG & %00110000
        Iorwf CCP3CON,f     ' CCP3CON = CCP3CON | WREG
        Bcf STATUS,0        ' CCPR3L = pDutyCycle >> 2
        Rrcf INDF0,w
        Bcf STATUS,0
        Rrcf WREG,f
        Movwf CCPR3L
    #endif
    #if(Prm_1 == Word) || (Prm_1 == Dword)
        Lfsr 0,(pDutyCycle)
        Movlw 207           ' CCP3CON = CCP3CON & %11001111
        Andwf CCP3CON,f
        Swapf POSTINC0,w    ' WREG = pDutyCycle << 4
        Andlw 48            ' WREG = WREG & %00110000
        Iorwf CCP3CON,f     ' CCP3CON = CCP3CON | WREG

        Rrcf POSTDEC0,w     ' CCPR3L = pDutyCycle >> 2
        Movwf PRODL,0
        Rrcf INDF0,w
        Movwf CCPR3L
        Rrcf PRODL,f
        Rrcf CCPR3L,f
    #endif
    #if(Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32)
        Movlw 207
        Andwf CCP3CON,f
        Movlw ((pDutyCycle << 4) & 48)
        Iorwf CCP3CON,f
        Movlw (pDutyCycle >> 2)
        Movwf CCPR3L
    #endif
    Reset_Bank          ' Back to RAM bank 0
Endm
$endif  ' _CCP3CON

$ifdef _CCP4CON                     ' Does the device contain the CCP4CON register?
'---------------------------------------------------------
$define WriteAnalog4(pDutyCycle) _mWriteAnalog4 pDutyCycle

_mWriteAnalog4 Macro pDutyCycle
    Movlb 15                ' CCP4 SFRs are in RAM bank 15
    #if(Prm_1 == Byte)
        Lfsr 0,(pDutyCycle)
        Movlw 207           ' CCP4CON = CCP4CON & %11001111
        Andwf CCP4CON,f
        Swapf INDF0,w       ' WREG = pDutyCycle << 4
        Andlw 48            ' WREG = WREG & %00110000
        Iorwf CCP4CON,f     ' CCP4CON = CCP4CON | WREG
        Bcf STATUS,0        ' CCPR4L = pDutyCycle >> 2
        Rrcf INDF0,w
        Bcf STATUS,0
        Rrcf WREG,f
        Movwf CCPR4L
    #endif
    #if(Prm_1 == Word) || (Prm_1 == Dword)
        Lfsr 0,(pDutyCycle)
        Movlw 207           ' CCP4CON = CCP4CON & %11001111
        Andwf CCP4CON,f
        Swapf POSTINC0,w    ' WREG = pDutyCycle << 4
        Andlw 48            ' WREG = WREG & %00110000
        Iorwf CCP4CON,f     ' CCP4CON = CCP4CON | WREG

        Rrcf POSTDEC0,w     ' CCPR4L = pDutyCycle >> 2
        Movwf PRODL,0
        Rrcf INDF0,w
        Movwf CCPR4L
        Rrcf PRODL,f
        Rrcf CCPR4L,f
    #endif
    #if(Prm_1 == Num8) || (Prm_1 == Num16) || (Prm_1 == Num32)
        Movlw 207
        Andwf CCP4CON,f
        Movlw ((pDutyCycle << 4) & 48)
        Iorwf CCP4CON,f
        Movlw (pDutyCycle >> 2)
        Movwf CCPR4L
    #endif
    Reset_Bank          ' Back to RAM bank 0
Endm
$endif  ' _CCP4CON
'---------------------------------------------------------
' Turn off PWM Module 1 by clearing CCP1CON
' Disable the PWM1 output pin
$define CloseAnalog1() CCP1CON = 0 : Input PORTC.2

'---------------------------------------------------------
' Turn off PWM Module 2 by clearing CCP2CON
' Disable the PWM2 output pin
$define CloseAnalog2() CCP2CON = 0 : Input PORTC.1

$ifdef _CCP3CON                     ' Does the device contain the CCP3CON register?
'---------------------------------------------------------
' Turn off PWM Module 3 by clearing CCP3CON
' Disable the PWM3 output pin
$define CloseAnalog3() CCP3CON = 0 : Input PORTB.5
$endif  ' _CCP3CON

$ifdef _CCP4CON                     ' Does the device contain the CCP4CON register?
'---------------------------------------------------------
' Turn off PWM Module 4 by clearing CCP4CON
' Disable the PWM4 output pin
$define CloseAnalog4() CCP4CON = 0 : Input PORTB.0
$endif  ' _CCP4CON

$endif  ' _HPWM10_INC_



